;;; -*- mode: lisp -*-
;;;
;;; lj.lisp --- 'library' of functions to interface with LJ.
;;;
;;; Copyright (C) 2001 by Mark Simpson
;;;
;;; Author: Mark Simpson <damned@rover>
;;; This program is free software; you can redistribute it and/or modify
;;; it under the terms of the GNU General Public License as published by
;;; the Free Software Foundation; either version 2, or (at your option)
;;; any later version.
;;;
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;; GNU General Public License for more details.
;;;
;;; You should have received a copy of the GNU General Public License
;;; along with this program; if not, you can either send email to this
;;; program's maintainer or write to: The Free Software Foundation,
;;; Inc.; 675 Massachusetts Avenue; Cambridge, MA 02139, USA.
;;;
;;; Commentary: 

;;; Code: 

(eval-when (:compile-toplevel :load-toplevel :execute)
  (require "lj-pkg" (logical-pathname "livejournal:lj-pkg"))
  (require "lj-util" (logical-pathname "livejournal:lj-util")))

(in-package "LJ")
  

;;;
;;; UTILITY FUNCTIONS
;;;
;;;
;;; LJ globals
;;;
(defparameter *server-host* "www.livejournal.com")
(defparameter *server-port* 80)
(defvar *username* "")
(defvar *password* "")

;;;
;;; LJ utilitiy functions
;;;
(defun make-data-str (mode vars)
  (format nil "mode=~A~{&~{~A=~A~}~}"
          mode
          (mapcar #'(lambda (x) (list (encode-str (first x))
                                      (encode-str (second x))))
                  vars)))

(defun send-msg (sock mode vars)
  (let ((str (make-data-str mode (delete nil vars))))
    (format sock "POST /cgi-bin/log.cgi HTTP/1.0~%~
                  Host: www.livejournal.com~%~
                  Content-type: multipart/form-data~%~
                  Content-length: ~D~%~
                  ~%~
                  ~A~%"
            (length str) str)))

(defun recv-msg (sock)
  (loop for line = (read-line sock nil nil)
        while line collect line))

(defun parse-msg (data-list)
  (let ((data (subseq data-list (1+ (position "" data-list
                                              :test #'string=))))
        (hash (make-hash-table :test 'equal)))
    (loop for key = (pop data)
          for value = (pop data)
          while key do (setf (gethash key hash) value))
    hash))

(defun do-msg (mode vars)
  "Returns hash table of values returned from server."
  (let ((hash (with-open-socket (sock (*server-host* *server-port*))
                                (send-msg sock mode vars)
                                (parse-msg (recv-msg sock)))))
    (when (string= (gethash "success" hash) "FAIL")
      (error (format t "FAILED: ~A" (gethash "errmsg" hash))))
    (when (not (gethash "success" hash))
      (error "Server error, try again later"))
    hash))

;;;
;;; LJ mode command defintion macro
;;;
(defmacro def-mode-cmd (mode (&body vars)
                             (&optional extra-vars-setup-form)
                             &body body)
  "DEF-MODE-CMD will generate a functioned called MODE which takes
variables VARS.  This function calls DO-MSG for \"mode\" with the variables
MODE_VARS.

  MODE_VARS is the concatenation of user, password and the result of
EXTRA-VARS-SETUP-FORM.

  After the DO-MSG is called BODY is evaluated in a PROGN is called.  The
hash table of values is available as the variable DATA.

  This generated function returns the hash table."
  `(defun ,mode ,vars
     (let* ((mode-vars (append
                        (list (list "user" *username*)
                              (list "password" *password*))
                        ,extra-vars-setup-form))
            (data (do-msg ,(format nil "~A"
                                   (string-downcase (symbol-name mode)))
                          mode-vars)))
       (progn ,@body)
       data)))

(def-mode-cmd login () (`(("clientversion" "Lisp-lj/0.1.0"))) ())

(def-mode-cmd postevent (subject text)
  ((multiple-value-bind (sec min hour dom mon year day tz dst)
       (get-decoded-time)
     (declare (ignore sec day tz dst))
     `(("event" ,text)
       ("subject" ,subject)
       ("year" ,(format nil "~4,'0D" year))
       ("mon" ,(format nil "~2,'0D" mon))
       ("day" ,(format nil "~2,'0D" dom))
       ("hour" ,(format nil "~2,'0D" hour))
       ("min" ,(format nil "~2,'0D" min)))))
  ())


(defvar *checkfriends-lastupdate* "")
(def-mode-cmd checkfriends
  (&optional (checkfriends-lastupdate *checkfriends-lastupdate*))
  (`(("lastupdate" ,checkfriends-lastupdate)))
  (setq *checkfriends-lastupdate* (gethash "lastupdate" data)))

(defvar *syncitems-lastupdate* "")
(def-mode-cmd syncitems 
  (&optional (syncitems-lastupdate *syncitems-lastupdate*))
  (`(("lastsync" ,syncitems-lastupdate)))
  ())

(def-mode-cmd getevents
  (&key (truncate nil)
        (subject-only nil)
        (noprops nil)
        (selecttype :one)
        (howmany 0)
        (year 0)
        (month 0)
        (day 0)
        (lastsync *syncitems-lastupdate*)
        (beforedate "")
        (itemid "-1")
        (lineendings "unix")
        (usejournal nil))
  (`(,(when truncate '("truncate" "1"))
     ,(when subject-only '("prefersubject" "1"))
     ,(when noprops '("noprops" "1"))
     ("selecttype" ,(ecase selecttype
                      (:one "one")
                      (:lastn "lastn")
                      (:day "day")
                      (:syncitems "syncitems")))
     ,@(ecase selecttype
         (:one (list (list "itemid" itemid)))
         (:lastn (list (if (> howmany 0)
                           (list "howmany"
                                 (if (< howmany 50) howmany 50))
                         (list "beforedate" beforedate))))
         (:day (list (list "year" (format nil "~4,'0D" year))
                     (list "mon" (format nil "~D" month))
                     (list "day" (format nil "~D" day))))
         (:syncitems (list (list "lastsync" lastsync))))
     ("lineendings" ,lineendings)
     ,(when usejournal (list "usejournal" usejournal))))
  ())

(def-mode-cmd getdaycounts
  (&key (usejournal nil))
  (`(,(when usejournal (list "usejournal" usejournal))))
  ())

(def-mode-cmd getfriendgroups () () ())
(def-mode-cmd friendof () () ())
(def-mode-cmd getfriends
  (&key (include-foaf nil)
        (include-groups nil))
  (`(,(when include-foaf (list "includefriendof" "1"))
     ,(when include-groups (list "includegroups" "1"))))
  ())

;;;
;;; LJ functions derived from mode functions
;;;
(defun post-file (file &optional (subject nil) (encode-html nil))
  (postevent (or subject file) (read-file file) encode-html))

(defun post-this-file ()
  "Assumes that this file is called lj.lisp and it is in the current
directory."
  (postevent (format nil "[GEEK Warning!] -- ~A" "lj.lisp")
             (format nil "<pre> ~A </pre>"
                     (encode-html (read-file "lj.lisp")))))


(defun init (username password
                      &key (server-host *server-host*)
                      (server-port *server-port*))
  (let ((make-changes-p nil))
    (let* ((*username* username)
           (*password* password)
           (*server-host* server-host)
           (*server-port* server-port)
           (data (ignore-errors (login))))
      (when (and data
                 (string= (gethash "success" data) "OK"))
        (format t "~&Welcome: ~A" (gethash "name" data))
        (setq make-changes-p t)))
    (if make-changes-p
        (progn
          (setq *username* username
                *password* password
                *server-host* server-host
                *server-port* server-port)
          t)
      nil)))


(provide 'lj)



